@@ -27,6 +27,7 @@ from asyncworker import AsyncWorker, AsyncItem
 from lib.constants import TVDB_API_KEY, DATA_DIR, DEFAULT_LANGUAGES
 from settings import Settings
 from datetime import datetime
+from datetime import timedelta
 from xml.sax import saxutils
 import gettext
 import locale
@@ -62,24 +63,21 @@ class Show(object):
         return [episode for episode in self.episode_list if episode.season_number == season_number]
 
     def get_next_episode(self, episode):
-        episodes = self.get_episodes_by_season(episode.season_number)
-        next_ep_number = episode.episode_number + 1
-        for ep in episodes:
-            if ep.episode_number == next_ep_number:
-                return ep
-        if episodes:
-            return episodes[0]
-        return None
+        return self._get_episode_by_offset(episode, 1)
 
     def get_previous_episode(self, episode):
-        episodes = self.get_episodes_by_season(episode.season_number)
-        next_ep_number = episode.episode_number - 1
-        for ep in episodes:
-            if ep.episode_number == next_ep_number:
-                return ep
-        if episodes:
-            return episodes[-1]
-        return None
+        return self._get_episode_by_offset(episode, -1)
+
+    def _get_episode_by_offset(self, episode, offset):
+        episodes = [(ep.episode_number, ep) for ep in \
+                    self.get_episodes_by_season(episode.season_number)]
+        episodes.sort()
+        number_of_episodes = len(episodes)
+        for i in range(number_of_episodes):
+            current_episode = episodes[i][1]
+            if current_episode == episode:
+                return episodes[(i + offset) % number_of_episodes][1]
+        return episode
 
     def get_seasons(self):
         seasons = []
@@ -123,8 +121,21 @@ class Show(object):
         for episode in episodes:
             self.delete_episode(episode)
 
-    def is_completely_watched(self):
-        for episode in self.episode_list:
+    def mark_all_episodes_as_not_watched(self, season = None):
+        self._mark_all_episodes(False, season)
+
+    def mark_all_episodes_as_watched(self, season = None):
+        self._mark_all_episodes(True, season)
+
+    def _mark_all_episodes(self, watched, season = None):
+        episodes = self.get_episodes_by_season(season)
+        for episode in episodes:
+            episode.watched = watched
+        SeriesManager().updated()
+
+    def is_completely_watched(self, season = None):
+        episodes = self.get_episodes_by_season(season)
+        for episode in episodes:
             if not episode.watched:
                 return False
         return True
@@ -141,16 +152,16 @@ class Show(object):
         info['episodes_to_watch'] = episodes_to_watch
         if season:
             sorted_episodes_to_watch = [('%02d'%int(episode.season_number), episode.episode_number, episode) \
-                                         for episode in episodes_to_watch]
+                                         for episode in episodes_to_watch if episode.air_date]
         else:
             # Leave Extras for end
             sorted_episodes_to_watch = [('%02d'%int(episode.season_number), episode.episode_number, episode) \
                                          for episode in episodes_to_watch \
-                                         if episode.season_number != '0']
+                                         if episode.season_number != '0' and episode.air_date]
             if not sorted_episodes_to_watch:
                 sorted_episodes_to_watch = [('%02d'%int(episode.season_number), episode.episode_number, episode) \
                                              for episode in episodes_to_watch \
-                                             if episode.season_number == '0']
+                                             if episode.season_number == '0' and episode.air_date]
         sorted_episodes_to_watch.sort()
         if sorted_episodes_to_watch:
             info['next_episode'] = sorted_episodes_to_watch[0][2]
@@ -185,7 +196,7 @@ class Show(object):
                     if next_episode:
                         next_air_date = next_episode.air_date
                         if next_air_date:
-                            show_info += ' | ' + _('<i>Next episode:</i> %s on %s') % \
+                            show_info += ' | ' + _('<i>Next episode:</i> %s, %s') % \
                                          (next_episode.get_episode_show_number(), \
                                          next_episode.get_air_date_text())
                         else:
@@ -225,7 +236,7 @@ class Show(object):
             if next_episode:
                 next_air_date = next_episode.air_date
                 if next_air_date:
-                    season_info += ' | ' + _('<i>Next episode:</i> %s on %s') % \
+                    season_info += ' | ' + _('<i>Next episode:</i> %s, %s') % \
                                    (next_episode.episode_number, \
                                     next_episode.get_air_date_text())
                 else:
@@ -310,6 +321,13 @@ class Episode(object):
     def get_air_date_text(self):
         if not self.air_date:
             return ''
+        today = datetime.today().date()
+        if today == self.air_date:
+            return _('Today')
+        if today + timedelta(days = 1) == self.air_date:
+            return _('Tomorrow')
+        if today - timedelta(days = 1) == self.air_date:
+            return _('Yesterday')
         next_air_date_str = self.air_date.strftime('%d %b')
         if self.air_date.year != datetime.today().year:
             next_air_date_str += self.air_date.strftime(' %Y')
@@ -431,6 +449,7 @@ class SeriesManager(gobject.GObject):
             self.thetvdb = thetvdbapi.TheTVDB(TVDB_API_KEY)
             self.async_worker = None
             self.changed = False
+            self.auto_save_id = None
 
             # Cached values
             self._cached_tvdb_shows = {}
@@ -563,7 +582,8 @@ class SeriesManager(gobject.GObject):
                                (show,),
                                None,)
         self.async_worker.queue.put((1, async_item))
-        self.async_worker.start()
+        if not self.async_worker.isAlive():
+            self.async_worker.start()
 
     def get_show_by_id(self, show_id):
         for show in self.series_list:
@@ -696,16 +716,19 @@ class SeriesManager(gobject.GObject):
 
     def save(self, save_file_path):
         if not self.changed:
-            return
+            return True
 
         dirname = os.path.dirname(save_file_path)
         if not (os.path.exists(dirname) and os.path.isdir(dirname)):
             os.mkdir(dirname)
         serialized = serializer.serialize(self.series_list)
-        save_file = open(save_file_path, 'w')
+        save_file_path_tmp = save_file_path + ".tmp"
+        save_file = open(save_file_path_tmp, 'w')
         save_file.write(serialized)
         save_file.close()
+        os.rename(save_file_path_tmp, save_file_path)
         self.changed = False
+        return True
 
     def load(self, file_path):
         if not os.path.exists(file_path):
@@ -722,3 +745,12 @@ class SeriesManager(gobject.GObject):
 
     def updated(self):
         self.changed = True
+
+    def auto_save(self, activate = True):
+        if activate and not self.auto_save_id:
+            self.auto_save_id = gobject.timeout_add(constants.SAVE_TIMEOUT_MS,
+                                                    self.save,
+                                                    constants.SF_DB_FILE)
+        elif self.auto_save_id and not activate:
+            gobject.source_remove(self.auto_save_id)
+            self.auto_save_id = None
